/*
    SPDX-FileCopyrightText: 2008 Nicola Gigante <nicola.gigante@gmail.com>
    SPDX-FileCopyrightText: 2009-2012 Dario Freddi <drf@kde.org>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#ifndef KAUTH_H
#define KAUTH_H

#include "kauthaction.h"
#include "kauthactionreply.h"
#include "kauthexecutejob.h"
#include "kauthhelpersupport.h"

#endif
