/*
    SPDX-FileCopyrightText: 2008 Nicola Gigante <nicola.gigante@gmail.com>
    SPDX-FileCopyrightText: 2009 Dario Freddi <drf@kde.org>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#include "HelperProxy.h"

namespace KAuth
{
HelperProxy::~HelperProxy()
{
}

} // namespace KAuth

#include "moc_HelperProxy.cpp"
